@@ -18,9 +18,11 @@ from contextlib import contextmanager
 from importlib import import_module
 from itertools import count
 from logging import getLogger
+from typing import Dict
 
 import getpass
 import os
+import pickle
 import socket
 import time
 import uuid
@@ -135,9 +137,7 @@ class DatabaseBackend(object):
         "{0.migration_table_quoted} "
         "ORDER by applied_at_utc"
     )
-    create_test_table_sql = (
-        "CREATE TABLE {table_name_quoted} " "(id INT PRIMARY KEY)"
-    )
+    create_test_table_sql = "CREATE TABLE {table_name_quoted} " "(id INT PRIMARY KEY)"
     log_migration_sql = (
         "INSERT INTO {0.log_table_quoted} "
         "(id, migration_hash, migration_id, operation, "
@@ -157,6 +157,7 @@ class DatabaseBackend(object):
     _is_locked = False
     _in_transaction = False
     _internal_schema_updated = False
+    _transactional_ddl_cache: Dict[bytes, bool] = {}
 
     def __init__(self, dburi, migration_table):
         self.uri = dburi
@@ -164,8 +165,16 @@ class DatabaseBackend(object):
         self._connection = self.connect(dburi)
         self.init_connection(self._connection)
         self.migration_table = migration_table
+        self.has_transactional_ddl = self._transactional_ddl_cache.get(
+            pickle.dumps(self.uri), True
+        )
+
+    def init_database(self):
         self.create_lock_table()
         self.has_transactional_ddl = self._check_transactional_ddl()
+        self._transactional_ddl_cache[
+            pickle.dumps(self.uri)
+        ] = self.has_transactional_ddl
 
     def _load_driver_module(self):
         """
@@ -192,6 +201,19 @@ class DatabaseBackend(object):
         db specific tasks required to make the connection ready for use.
         """
 
+    def copy(self):
+        """
+        Return a copy of the backend with a independent db
+        connection.
+        """
+        return self.__class__(self.uri, self.migration_table)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.connection.close()
+
     def __getattr__(self, attrname):
         if attrname.endswith("_quoted"):
             unquoted = getattr(self, attrname.rsplit("_quoted")[0])
@@ -211,9 +233,7 @@ class DatabaseBackend(object):
         """
         table_name = "yoyo_tmp_{}".format(utils.get_random_string(10))
         table_name_quoted = self.quote_identifier(table_name)
-        sql = self.create_test_table_sql.format(
-            table_name_quoted=table_name_quoted
-        )
+        sql = self.create_test_table_sql.format(table_name_quoted=table_name_quoted)
         with self.transaction() as t:
             self.execute(sql)
             t.rollback()
@@ -317,9 +337,7 @@ class DatabaseBackend(object):
                 with self.transaction():
                     self.execute(
                         "INSERT INTO {} (locked, ctime, pid) "
-                        "VALUES (1, :when, :pid)".format(
-                            self.lock_table_quoted
-                        ),
+                        "VALUES (1, :when, :pid)".format(self.lock_table_quoted),
                         {"when": datetime.utcnow(), "pid": pid},
                     )
             except self.DatabaseError:
@@ -331,9 +349,7 @@ class DatabaseBackend(object):
                     if row:
                         raise exceptions.LockTimeout(
                             "Process {} has locked this database "
-                            "(run yoyo break-lock to remove this lock)".format(
-                                row[0]
-                            )
+                            "(run yoyo break-lock to remove this lock)".format(row[0])
                         )
                     else:
                         raise exceptions.LockTimeout(
@@ -368,9 +384,7 @@ class DatabaseBackend(object):
             raise TypeError("Expected dict or other mapping object")
 
         cursor = self.cursor()
-        sql, params = utils.change_param_style(
-            self.driver.paramstyle, sql, params
-        )
+        sql, params = utils.change_param_style(self.driver.paramstyle, sql, params)
         cursor.execute(sql, params)
         return cursor
 
@@ -489,7 +503,8 @@ class DatabaseBackend(object):
         """
         logger.info("Applying %s", migration.id)
         self.ensure_internal_schema_updated()
-        migration.process_steps(self, "apply", force=force)
+        with self.copy() as migration_backend:
+            migration.process_steps(migration_backend, "apply", force=force)
         self.log_migration(migration, "apply")
         if mark:
             with self.transaction():
@@ -501,7 +516,8 @@ class DatabaseBackend(object):
         """
         logger.info("Rolling back %s", migration.id)
         self.ensure_internal_schema_updated()
-        migration.process_steps(self, "rollback", force=force)
+        with self.copy() as migration_backend:
+            migration.process_steps(migration_backend, "rollback", force=force)
         self.log_migration(migration, "rollback")
         with self.transaction():
             self.unmark_one(migration, log=False)
@@ -655,8 +671,12 @@ class SQLiteBackend(DatabaseBackend):
     list_tables_sql = "SELECT name FROM sqlite_master WHERE type = 'table'"
 
     def connect(self, dburi):
+        # Ensure that multiple connections share the same data
+        # https://sqlite.org/sharedcache.html
         conn = self.driver.connect(
-            dburi.database, detect_types=self.driver.PARSE_DECLTYPES
+            f"file:{dburi.database}?cache=shared",
+            uri=True,
+            detect_types=self.driver.PARSE_DECLTYPES,
         )
         conn.isolation_level = None
         return conn
@@ -733,18 +753,14 @@ class RedshiftBackend(PostgresqlBackend):
                 if not row:
                     self.execute(
                         "INSERT INTO {} (locked, ctime, pid) "
-                        "VALUES (1, :when, :pid)".format(
-                            self.lock_table_quoted
-                        ),
+                        "VALUES (1, :when, :pid)".format(self.lock_table_quoted),
                         {"when": datetime.utcnow(), "pid": pid},
                     )
                     return
                 elif timeout and time.time() > started + timeout:
                     raise exceptions.LockTimeout(
                         "Process {} has locked this database "
-                        "(run yoyo break-lock to remove this lock)".format(
-                            row[0]
-                        )
+                        "(run yoyo break-lock to remove this lock)".format(row[0])
                     )
                 else:
                     time.sleep(poll_interval)
@@ -762,7 +778,7 @@ class SnowflakeBackend(DatabaseBackend):
             account=dburi.hostname,
             database=database,
             schema=schema,
-            warehouse=dburi.args["warehouse"],
+            **dburi.args,
         )
 
     def savepoint(self, id):
