@@ -26,108 +26,107 @@ from yoyo import read_migrations
 from yoyo import exceptions
 from yoyo import ancestors, descendants
 
-from yoyo.tests import with_migrations, migrations_dir, dburi
+from yoyo.tests import migrations_dir
 from yoyo.tests import tempdir
 from yoyo.migrations import topological_sort, MigrationList
 from yoyo.scripts import newmigration
 
 
-@with_migrations(
-    """
-    step("CREATE TABLE yoyo_test (id INT)")
-    """,
-    """
-step("INSERT INTO yoyo_test VALUES (1)")
-step("INSERT INTO yoyo_test VALUES ('x', 'y')")
-    """,
-)
-def test_transaction_is_not_committed_on_error(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    with pytest.raises(backend.DatabaseError):
-        backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT count(1) FROM yoyo_test")
-    assert cursor.fetchone() == (0,)
+def test_transaction_is_not_committed_on_error(backend_sqlite3):
+    with migrations_dir(
+        'step("CREATE TABLE yoyo_test (id INT)")',
+        """
+        step("INSERT INTO yoyo_test VALUES (1)")
+        step("INSERT INTO yoyo_test VALUES ('x', 'y')")
+        """,
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        with pytest.raises(backend_sqlite3.DatabaseError):
+            backend_sqlite3.apply_migrations(migrations)
+        backend_sqlite3.rollback()
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT count(1) FROM yoyo_test")
+        assert cursor.fetchone() == (0,)
 
 
-@with_migrations(
-    'step("CREATE TABLE yoyo_test (id INT)")',
-    """
-step("INSERT INTO yoyo_test VALUES (1)", "DELETE FROM yoyo_test WHERE id=1")
-step("UPDATE yoyo_test SET id=2 WHERE id=1", "UPDATE yoyo_test SET id=1 WHERE id=2")
-    """,
-)
-def test_rollbacks_happen_in_reverse(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == [(2,)]
-    backend.rollback_migrations(migrations)
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == []
+def test_rollbacks_happen_in_reverse(backend_sqlite3):
+    with migrations_dir(
+        'step("CREATE TABLE yoyo_test (id INT)")',
+        """
+        step(
+            "INSERT INTO yoyo_test VALUES (1)", "DELETE FROM yoyo_test WHERE id=1"
+        )
+        step(
+            "UPDATE yoyo_test SET id=2 WHERE id=1",
+            "UPDATE yoyo_test SET id=1 WHERE id=2"
+        )
+        """,
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == [(2,)]
+        backend_sqlite3.rollback_migrations(migrations)
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == []
 
 
-@with_migrations(
-    """
-    step("CREATE TABLE yoyo_test (id INT)")
-    step("INSERT INTO yoyo_test VALUES (1)")
-    step("INSERT INTO yoyo_test VALUES ('a', 'b')", ignore_errors='all')
-    step("INSERT INTO yoyo_test VALUES (2)")
-    """
-)
-def test_execution_continues_with_ignore_errors(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == [(1,), (2,)]
+def test_execution_continues_with_ignore_errors(backend_sqlite3):
+    with migrations_dir(
+        """
+        step("CREATE TABLE yoyo_test (id INT)")
+        step("INSERT INTO yoyo_test VALUES (1)")
+        step("INSERT INTO yoyo_test VALUES ('a', 'b')", ignore_errors='all')
+        step("INSERT INTO yoyo_test VALUES (2)")
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == [(1,), (2,)]
 
 
-@with_migrations(
-    """
-    from yoyo import step, group
-    step("CREATE TABLE yoyo_test (id INT)")
-    group(
-        step("INSERT INTO yoyo_test VALUES (1)"),
-        step("INSERT INTO yoyo_test VALUES ('a', 'b')"),
-        ignore_errors='all'
-    )
-    step("INSERT INTO yoyo_test VALUES (2)")
-    """
-)
-def test_execution_continues_with_ignore_errors_in_transaction(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == [(2,)]
+def test_execution_continues_with_ignore_errors_in_transaction(backend_sqlite3):
+    with migrations_dir(
+        """
+        from yoyo import step, group
+        step("CREATE TABLE yoyo_test (id INT)")
+        group(
+            step("INSERT INTO yoyo_test VALUES (1)"),
+            step("INSERT INTO yoyo_test VALUES ('a', 'b')"),
+            ignore_errors='all'
+        )
+        step("INSERT INTO yoyo_test VALUES (2)")
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == [(2,)]
 
 
-@with_migrations(
-    """
-    step("CREATE TABLE yoyo_test (id INT)")
-    step("INSERT INTO yoyo_test VALUES (1)",
-         "DELETE FROM yoyo_test WHERE id=2")
-    step("UPDATE yoyo_test SET id=2 WHERE id=1",
-         "SELECT nonexistent FROM imaginary", ignore_errors='rollback')
-    """
-)
-def test_rollbackignores_errors(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == [(2,)]
+def test_rollbackignores_errors(backend_sqlite3):
+    with migrations_dir(
+        """
+        step("CREATE TABLE yoyo_test (id INT)")
+        step("INSERT INTO yoyo_test VALUES (1)",
+            "DELETE FROM yoyo_test WHERE id=2")
+        step("UPDATE yoyo_test SET id=2 WHERE id=1",
+            "SELECT nonexistent FROM imaginary", ignore_errors='rollback')
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == [(2,)]
 
-    backend.rollback_migrations(migrations)
-    cursor.execute("SELECT * FROM yoyo_test")
-    assert cursor.fetchall() == []
+        backend_sqlite3.rollback_migrations(migrations)
+        cursor.execute("SELECT * FROM yoyo_test")
+        assert cursor.fetchall() == []
 
 
 def test_migration_is_committed(backend):
@@ -135,19 +134,20 @@ def test_migration_is_committed(backend):
         migrations = read_migrations(tmpdir)
         backend.apply_migrations(migrations)
 
-    backend.rollback()
-    rows = backend.execute("SELECT * FROM yoyo_test").fetchall()
-    assert list(rows) == []
+        backend.rollback()
+        rows = backend.execute("SELECT * FROM yoyo_test").fetchall()
+        assert list(rows) == []
 
 
 def test_rollback_happens_on_step_failure(backend):
     with migrations_dir(
         """
-                        step("",
-                             "CREATE TABLE yoyo_is_rolledback (i INT)"),
-                        step("CREATE TABLE yoyo_test (s VARCHAR(100))",
-                             "DROP TABLE yoyo_test")
-                        step("invalid sql!")"""
+        step("",
+                "CREATE TABLE yoyo_is_rolledback (i INT)"),
+        step("CREATE TABLE yoyo_test (s VARCHAR(100))",
+                "DROP TABLE yoyo_test")
+        step("invalid sql!")
+        """
     ) as tmpdir:
         migrations = read_migrations(tmpdir)
         with pytest.raises(backend.DatabaseError):
@@ -169,87 +169,84 @@ def test_rollback_happens_on_step_failure(backend):
         assert list(cursor.fetchall()) == []
 
 
-@with_migrations(
-    """
-    step("CREATE TABLE yoyo_test (id INT)")
-    step("DROP TABLE yoyo_test")
-    """
-)
-def test_specify_migration_table(tmpdir):
-    backend = get_backend(dburi, migration_table="another_migration_table")
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT migration_id FROM another_migration_table")
-    assert cursor.fetchall() == [("0",)]
+def test_specify_migration_table(tmpdir, dburi):
+    with migrations_dir(
+        """
+        step("CREATE TABLE yoyo_test (id INT)")
+        step("DROP TABLE yoyo_test")
+        """
+    ) as tmpdir:
+        backend = get_backend(dburi, migration_table="another_migration_table")
+        migrations = read_migrations(tmpdir)
+        backend.apply_migrations(migrations)
+        cursor = backend.cursor()
+        cursor.execute("SELECT migration_id FROM another_migration_table")
+        assert list(cursor.fetchall()) == [("0",)]
 
 
-@with_migrations(
-    """
-    def foo(conn):
-        conn.cursor().execute("CREATE TABLE foo_test (id INT)")
-        conn.cursor().execute("INSERT INTO foo_test VALUES (1)")
-    def bar(conn):
-        foo(conn)
-    step(bar)
-    """
-)
-def test_migration_functions_have_namespace_access(tmpdir):
+def test_migration_functions_have_namespace_access(backend_sqlite3):
     """
     Test that functions called via step have access to the script namespace
     """
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT id FROM foo_test")
-    assert cursor.fetchall() == [(1,)]
+    with migrations_dir(
+        """
+        def foo(conn):
+            conn.cursor().execute("CREATE TABLE foo_test (id INT)")
+            conn.cursor().execute("INSERT INTO foo_test VALUES (1)")
+        def bar(conn):
+            foo(conn)
+        step(bar)
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT id FROM foo_test")
+        assert cursor.fetchall() == [(1,)]
 
 
-@with_migrations(
-    """
-    from yoyo import group, step
-    step("CREATE TABLE yoyo_test (id INT)")
-    group(step("INSERT INTO yoyo_test VALUES (1)")),
-    """
-)
-def test_migrations_can_import_step_and_group(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    backend.apply_migrations(migrations)
-    cursor = backend.cursor()
-    cursor.execute("SELECT id FROM yoyo_test")
-    assert cursor.fetchall() == [(1,)]
+def test_migrations_can_import_step_and_group(backend_sqlite3):
+    with migrations_dir(
+        """
+        from yoyo import group, step
+        step("CREATE TABLE yoyo_test (id INT)")
+        group(step("INSERT INTO yoyo_test VALUES (1)")),
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        backend_sqlite3.apply_migrations(migrations)
+        cursor = backend_sqlite3.cursor()
+        cursor.execute("SELECT id FROM yoyo_test")
+        assert cursor.fetchall() == [(1,)]
 
 
-@with_migrations(
-    """
-    step("CREATE TABLE yoyo_test (id INT, c VARCHAR(1))")
-    step("INSERT INTO yoyo_test VALUES (1, 'a')")
-    step("INSERT INTO yoyo_test VALUES (2, 'b')")
-    step("SELECT * FROM yoyo_test")
-    """
-)
-def test_migrations_display_selected_data(tmpdir):
-    backend = get_backend(dburi)
-    migrations = read_migrations(tmpdir)
-    with patch("yoyo.migrations.sys.stdout") as stdout:
-        backend.apply_migrations(migrations)
-        written = "".join(a[0] for a, kw in stdout.write.call_args_list)
-        assert written == (
-            " id | c \n" "----+---\n" " 1  | a \n" " 2  | b \n" "(2 rows)\n"
-        )
+def test_migrations_display_selected_data(backend_sqlite3):
+    with migrations_dir(
+        """
+        step("CREATE TABLE yoyo_test (id INT, c VARCHAR(1))")
+        step("INSERT INTO yoyo_test VALUES (1, 'a')")
+        step("INSERT INTO yoyo_test VALUES (2, 'b')")
+        step("SELECT * FROM yoyo_test")
+        """
+    ) as tmpdir:
+        migrations = read_migrations(tmpdir)
+        with patch("yoyo.migrations.sys.stdout") as stdout:
+            backend_sqlite3.apply_migrations(migrations)
+            written = "".join(a[0] for a, kw in stdout.write.call_args_list)
+            assert written == (
+                " id | c \n" "----+---\n" " 1  | a \n" " 2  | b \n" "(2 rows)\n"
+            )
 
 
 def test_grouped_migrations_can_be_rolled_back(backend):
-    with with_migrations(
+    with migrations_dir(
         a="from yoyo import step\n"
         'steps = [step("CREATE TABLE p (n INT)",'
         '              "DROP TABLE p")]'
     ) as t1:
         backend.apply_migrations(read_migrations(t1))
 
-        with with_migrations(
+        with migrations_dir(
             b="from yoyo import group\n"
             "from yoyo import step\n"
             "steps = [\n"
@@ -268,6 +265,7 @@ def test_grouped_migrations_can_be_rolled_back(backend):
             backend.rollback_migrations(migrations)
             cursor = backend.execute("SELECT count(1) FROM p")
             assert cursor.fetchone() == (0,)
+            backend.rollback()
         backend.rollback_migrations(read_migrations(t1))
 
 
@@ -417,47 +415,65 @@ class TestAncestorsDescendants(object):
 
 
 class TestReadMigrations(object):
-    @with_migrations(**{newmigration.tempfile_prefix + "test": ""})
-    def test_it_ignores_yoyo_new_tmp_files(self, tmpdir):
+    def test_it_ignores_yoyo_new_tmp_files(self):
         """
         The yoyo new command creates temporary files in the migrations directory.
         These shouldn't be picked up by yoyo apply etc
         """
-        assert len(read_migrations(tmpdir)) == 0
+        with migrations_dir(
+            **{newmigration.tempfile_prefix + "test": ""}
+        ) as tmpdir:
+            assert len(read_migrations(tmpdir)) == 0
 
-    @with_migrations(**{"post-apply": """step('SELECT 1')"""})
-    def test_it_loads_post_apply_scripts(self, tmpdir):
-        migrations = read_migrations(tmpdir)
-        assert len(migrations) == 0
+    def test_it_loads_post_apply_scripts(self):
+        with migrations_dir(**{"post-apply": "step('SELECT 1')"}) as tmpdir:
+            migrations = read_migrations(tmpdir)
+            assert len(migrations) == 0
         assert len(migrations.post_apply) == 1
 
-    @with_migrations(**{"a": """step('SELECT 1')"""})
-    def test_it_does_not_add_duplicate_steps(self, tmpdir):
-        m = read_migrations(tmpdir)[0]
-        m.load()
-        assert len(m.steps) == 1
+    def test_it_does_not_add_duplicate_steps(self):
+        with migrations_dir("step('SELECT 1')") as tmpdir:
+            m = read_migrations(tmpdir)[0]
+            m.load()
+            assert len(m.steps) == 1
 
-        m = read_migrations(tmpdir)[0]
-        m.load()
-        assert len(m.steps) == 1
+            m = read_migrations(tmpdir)[0]
+            m.load()
+            assert len(m.steps) == 1
 
-    @with_migrations(**{"a": """from yoyo import step; step('SELECT 1')"""})
     def test_it_does_not_add_duplicate_steps_with_imported_symbols(
         self, tmpdir
     ):
-        m = read_migrations(tmpdir)[0]
-        m.load()
-        assert len(m.steps) == 1
+        with migrations_dir(
+            a="from yoyo import step; step('SELECT 1')"
+        ) as tmpdir:
+            m = read_migrations(tmpdir)[0]
+            m.load()
+            assert len(m.steps) == 1
 
-        m = read_migrations(tmpdir)[0]
-        m.load()
-        assert len(m.steps) == 1
+            m = read_migrations(tmpdir)[0]
+            m.load()
+            assert len(m.steps) == 1
 
     def test_it_reads_from_package_data(self):
         migrations = read_migrations("package:yoyo:tests/migrations")
         assert len(migrations) == 1
         assert migrations[0].id == "test-pkg-migration"
 
+    def test_it_reads_relative_path(self):
+        """
+        https://todo.sr.ht/~olly/yoyo/79
+        """
+        with migrations_dir(a="from yoyo import step; step('SELECT 1')") as tmpdir:
+            saved_cwd = os.getcwd()
+            try:
+                os.chdir(tmpdir)
+                m = read_migrations(".")[0]
+                m.load()
+                assert len(m.steps) == 1
+            finally:
+                os.chdir(saved_cwd)
+
     def test_it_globs_directory_names(self):
         def touch(f):
             io.open(f, "w").close()
@@ -535,13 +551,14 @@ class TestReadMigrations(object):
         with pytest.raises(exceptions.BadMigration):
             check("-- depends: true\nSELECT 1", set())
 
-    def test_it_does_not_mix_up_migrations_from_different_sources(self):
-        backend = get_backend(dburi)
+    def test_it_does_not_mix_up_migrations_from_different_sources(
+        self, backend_sqlite3
+    ):
         with migrations_dir(**{"1.sql": "", "3.sql": ""}) as t1, migrations_dir(
             **{"2.sql": "", "4.sql": ""}
         ) as t2:
             migrations = read_migrations(t1, t2)
-            assert [m.id for m in backend.to_apply(migrations)] == [
+            assert [m.id for m in backend_sqlite3.to_apply(migrations)] == [
                 "1",
                 "3",
                 "2",
@@ -550,9 +567,7 @@ class TestReadMigrations(object):
 
 
 class TestPostApplyHooks(object):
-    def test_post_apply_hooks_are_run_every_time(self):
-
-        backend = get_backend(dburi)
+    def test_post_apply_hooks_are_run_every_time(self, backend_sqlite3):
         migrations = migrations_dir(
             **{
                 "a": "step('create table postapply (i int)')",
@@ -563,12 +578,14 @@ class TestPostApplyHooks(object):
         with migrations as tmp:
 
             def count_postapply_calls():
-                cursor = backend.cursor()
+                cursor = backend_sqlite3.cursor()
                 cursor.execute("SELECT count(1) FROM postapply")
                 return cursor.fetchone()[0]
 
             def _apply_migrations():
-                backend.apply_migrations(backend.to_apply(read_migrations(tmp)))
+                backend_sqlite3.apply_migrations(
+                    backend_sqlite3.to_apply(read_migrations(tmp))
+                )
 
             # Should apply migration 'a' and call the post-apply hook
             _apply_migrations()
@@ -583,32 +600,34 @@ class TestPostApplyHooks(object):
             _apply_migrations()
             assert count_postapply_calls() == 2
 
-    @with_migrations(
-        **{
-            "a": "step('create table postapply (i int)')",
-            "post-apply": "step('insert into postapply values (1)')",
-            "post-apply2": "step('insert into postapply values (2)')",
-        }
-    )
-    def test_it_runs_multiple_post_apply_hooks(self, tmpdir):
-        backend = get_backend(dburi)
-        backend.apply_migrations(backend.to_apply(read_migrations(tmpdir)))
-        cursor = backend.cursor()
-        cursor.execute("SELECT * FROM postapply")
-        assert cursor.fetchall() == [(1,), (2,)]
+    def test_it_runs_multiple_post_apply_hooks(self, backend_sqlite3):
+        with migrations_dir(
+            **{
+                "a": "step('create table postapply (i int)')",
+                "post-apply": "step('insert into postapply values (1)')",
+                "post-apply2": "step('insert into postapply values (2)')",
+            }
+        ) as tmpdir:
+            backend_sqlite3.apply_migrations(
+                backend_sqlite3.to_apply(read_migrations(tmpdir))
+            )
+            cursor = backend_sqlite3.cursor()
+            cursor.execute("SELECT * FROM postapply")
+            assert cursor.fetchall() == [(1,), (2,)]
 
-    @with_migrations(
-        **{
-            "a": "step('create table postapply (i int)')",
-            "post-apply": "step('insert into postapply values (1)')",
-        }
-    )
-    def test_apply_migrations_only_does_not_run_hooks(self, tmpdir):
-        backend = get_backend(dburi)
-        backend.apply_migrations_only(backend.to_apply(read_migrations(tmpdir)))
-        cursor = backend.cursor()
-        cursor.execute("SELECT * FROM postapply")
-        assert cursor.fetchall() == []
+    def test_apply_migrations_only_does_not_run_hooks(self, backend_sqlite3):
+        with migrations_dir(
+            **{
+                "a": "step('create table postapply (i int)')",
+                "post-apply": "step('insert into postapply values (1)')",
+            }
+        ) as tmpdir:
+            backend_sqlite3.apply_migrations_only(
+                backend_sqlite3.to_apply(read_migrations(tmpdir))
+            )
+            cursor = backend_sqlite3.cursor()
+            cursor.execute("SELECT * FROM postapply")
+            assert cursor.fetchall() == []
 
 
 class TestLogging(object):
@@ -628,7 +647,7 @@ class TestLogging(object):
         return backend.execute("SELECT count(1) FROM _yoyo_log").fetchone()[0]
 
     def test_it_logs_apply_and_rollback(self, backend):
-        with with_migrations(
+        with migrations_dir(
             a='step("CREATE TABLE yoyo_test (id INT)")'
         ) as tmpdir:
             migrations = read_migrations(tmpdir)
@@ -650,7 +669,7 @@ class TestLogging(object):
             assert logged["created_at_utc"] >= apply_time
 
     def test_it_logs_mark_and_unmark(self, backend):
-        with with_migrations(
+        with migrations_dir(
             a='step("CREATE TABLE yoyo_test (id INT)")'
         ) as tmpdir:
             migrations = read_migrations(tmpdir)
